from graphics import *
from time import sleep
from math import * 

class Bug:
    "Represents a graphical bug."
    
    def __init__(self, center, radius):
        """ 
        Create a new bug 
        input:
        center: The center Point of the largest Circle in the bug
        radius: The radius of the largest Circle in the bug
        """
        c1 = Circle(center, radius)
        c1.setFill("red")
    
        dX = center.getX()
        dY = center.getY()
        p1 = Point((dX+radius/5.0), dY+radius)
    
        eye = Circle(p1,radius/5)
        eye.setFill("yellow")
    
        eye1 = eye.clone()
        eye1.move(-radius/2.5,0)
        eyeball = Circle(p1, radius/10)
        eyeball.setFill("black")
    
        eyeball1 = eyeball.clone()
        eyeball1.move(-radius/2.5, 0)
    
        self.bugComponents = [c1, eye, eye1, eyeball, eyeball1]

    
    def draw(self, window):
        """
        Draw the bug in a given window
        Parameter:
        window: A GraphWin object in which to draw the bug
        """
        for comp in self.bugComponents:
            comp.draw(window)
      

    def move(self, dx, dy):
        """ 
        Move a bug by a specified amount 
        Parameters:
        dx: The distance to move in the x direction
        dy: The distance to move in the y direction
        """
        for comp in self.bugComponents:
            comp.move(dx,dy)


    def walk(self, dx, dy, seconds):
        """
        Move a bug by a specified amount in a specified time
        Parameters -
        dx: The total distance to move in the x direction
        dy: The total distance to move in the y direction
        steps: the number of total steps to take
        """
        STEPS_PER_SECOND=20
        
        steps = seconds*STEPS_PER_SECOND # total number of steps
        for i in range(steps):
            self.move(dx/steps, dy/steps)
            # need to reduce the sleep; otherwise, sleeps too long;
            # the reduction helps to account for the move time 
            sleep(seconds/steps*.1) 

    def rotate(self, degrees):
        """
        Rotate a bug a specified number of degrees
        Parameters - 
        degrees: the number of degrees to rotate the bug
        """
    
        r = radians(degrees)
    
        circX1 = self.bugComponents[0].getCenter().getX()
        circY1 = self.bugComponents[0].getCenter().getY()
    
        for i in self.bugComponents:
            circX2 = i.getCenter().getX()
            circY2 = i.getCenter().getY()
            dx = circX2 - circX1
            dy = circY2 - circY1
            mx = dx*(cos(r)-1) - dy*sin(r)
            my = dx* sin(r) + dy*(cos(r)-1)
            i.move(mx,my)


    def setColor(self, color):
        """
        Change the color of the bug during the animation
        Parameters - 
        color: desired color change
        """
        # we know the first item in the list is the largest segment
        self.bugComponents[0].setFill(color)


def main():
    # create a window for the bugs
    window = GraphWin("Busy Bugs!",600,600)
    window.setBackground("black")
    window.setCoords(0,0,600,600)
    
    # create 4 bugs at various places in the window (using an appropriate
    # function)
    # put the bugs in a list

    bug1 = Bug(Point(50,60),30)
    bug2 = Bug(Point(200,350), 50)
    bug3 = Bug(Point(350,50), 20)
    bug4 = Bug(Point(350,300), 60)

    bugs = [bug1, bug2, bug3, bug4] # bug is defined as a list of the 4 bugs
    
    # draw the bugs (using an appropriate function)
    
    for n in range(4):
        bugs[n].draw(window)

    window.getMouse()

    # rotate one bug, pause

    for i in range(5):
        bug2.rotate(30)
        sleep(0.05)
        
        
    # change the color of one bug    
    for i in range(4):
        bugs[i].setColor(color_rgb(i*255/4, 255, 255))
        
    bugs[3].move(10, 10)
        
    bugs[0].walk(100, 100, 2)
    
    # wait for the user to click, then close the window
    window.getMouse()
    window.close()
   
main()


