class Card:
    """
    A class to represent a standard playing card. The ranks are ints:
    2-10 for numbered cards, 11=Jack, 12=Queen, 13=King, 14=Ace.
    The suits are strings: 'clubs', 'spades', 'hearts', 'diamonds'.
    """
    
    def __init__(self, rank, suit):
        "Constructs a new Card object with the given rank (an int) and suit (a string)."
        self.rank = rank
        self.suit = suit

    def __str__(self):
        "Returns a string describing the card as 'rank of suit'."
        result = ""
        if self.rank == 11:
            result += "Jack"
        elif self.rank == 12:
            result += "Queen"
        elif self.rank == 13:
            result += "King"
        elif self.rank == 14:
            result += "Ace"
        else:
            result += str(self.rank)
        result += " of " + self.suit
        return result
        
    def getRank(self):
        "Returns rank."	
        return self.rank

    def getSuit(self):
        "Returns suit."
        return self.suit
        
    def rummyValue(self):
        "Returns the card's rummy value"
        if self.getRank() < 10: # handles number cards
            return 5
        elif self.rank == 14: # handles Ace
            return 15
        else: # handles 10 and face cards
            return 10
            
    def __eq__(self, other):
        """Returns true if this object and other are equivalent
        based on ..."""
        
        # verify that self and other are the same type
        if type(self) != type(other):
            return False
        # do comparison
        return self.rank == other.rank and self.suit == other.suit
        
    def __lt__(self, other):
        """ Compares Card objects by """
        
        # verify that self and other are the same type
        
        # do comparison
        return self.rank < other.rank

        
def main():
    c1 = Card(14, "spades")
    print(c1)
    c2 = Card(13, "hearts")
    print(c2)
    c3 = Card(2, "diamonds")
    print(c3)
    
    # testing rummyValue() method
    rummyValue = c1.rummyValue()
    print("Rummy value is", end = " ")
    if rummyValue == 15:
        print("good", end = " ")
    else:
        print("bad", end = " ")
    print("for", c1)
    
    testCases = [c1, c2, c3]
    expected = [15, 10, 5]

    for pos in range(len(testCases)):
        testCase = testCases[pos] # what data type is this?
        actualVal = testCase.rummyValue()
        if actualVal == expected[pos]:
            print("Success for rummyVal on", testCase, "-->", actualVal)
    
    # test equals
    result = c1 == "a string"
    print(result, "should be False")
    
    result = c1 == c2
    print(result, "should be False")
    
    # test lt
    #result = c1 < "a string"
    #print(result, "should be False")
    
    result = c1 < c2
    print(result, "should be False")
    # 
    result = c2 < c1
    print(result, "should be True")
    
    
# Since I am probably going to import this script into another script,
# I only want to call main() when it's *not* imported 
if __name__ == '__main__':
    main()
