# This is a program that displays a window with moving bugs constructed by
# calling functions.
# YOUR_NAME_HERE

from graphics import *
from time import sleep
from math import * 

def buildBug(center, radius):
    """ 
    Create a new bug
    input:
    center: The center Point of the largest Circle in the bug
    radius: The radius of the largest Circle in the bug
    
    return: A list of Circle objects describing the bug, with the largest
    Circle first
    """
    c1 = Circle(center, radius)
    c1.setFill("red")

    dX = center.getX()
    dY = center.getY()
    p1 = Point((dX+radius/5.0), dY+radius)

    eye = Circle(p1,radius/5)
    eye.setFill("yellow")

    eye1 = eye.clone()
    eye1.move(-radius/2.5,0)
    eyeball = Circle(p1, radius/10)
    eyeball.setFill("black")

    eyeball1 = eyeball.clone()
    eyeball1.move(-radius/2.5, 0)

    bug = [c1, eye, eye1, eyeball, eyeball1]

    return bug

    
def drawBug(bug, window):
    """
    Draw a bug in a given window
    Parameters:
    bug: A list of Circle objects 
    window: A GraphWin object in which to draw the bug
    """
    for item in bug:
        item.draw(window)
      

def moveBug(bug, dx, dy):
    """ 
    Move a bug by a specified amount 
    Parameters:
    bug: A list of Circle objects describing a bug
    dx: The distance to move in the x direction
    dy: The distance to move in the y direction
    """
    for i in bug:
        i.move(dx,dy)


def walkBug(bug, dx, dy, seconds):
    """
    Move a bug by a specified amount in a specified time
    Parameters -
    bug: A list of Circle objects describing a bug
    dx: The total distance to move in the x direction
    dy: The total distance to move in the y direction
    steps: the number of total steps to take
    wait: the time to wait between steps
    """
    STEPS_PER_SECOND=20
    
    steps = seconds*STEPS_PER_SECOND # total number of steps
    for i in range(steps):
        moveBug(bug, dx/steps, dy/steps)
        # need to reduce the sleep; otherwise, sleeps too long;
        # the reduction helps to account for the move time 
        sleep(seconds/steps*.1) 

def rotateBug(bug, degrees):
    """
    Rotate a bug a specified number of degrees
    Parameters - 
    bug: A list of Circle objects describing a bug
    degrees: the number of degrees to rotate the bug
    """

    r = radians(degrees)

    circX1 = bug[0].getCenter().getX()
    circY1 = bug[0].getCenter().getY()

    for i in bug:
        circX2 = i.getCenter().getX()
        circY2 = i.getCenter().getY()
        dx = circX2 - circX1
        dy = circY2 - circY1
        mx = dx*(cos(r)-1) - dy*sin(r)
        my = dx* sin(r) + dy*(cos(r)-1)
        i.move(mx,my)


def setBugColor(bug, color):
    """
    Change the color of the bug during the animation
    Parameters - 
    bug: A list of circle objects representing a bug
    color: desired color change
    """
    # we know the first item in the list is the largest segment
    bug[0].setFill(color)


def main():
    # create a window for the bugs
    window = GraphWin("Busy Bugs!",600,600)
    window.setBackground("black")
    window.setCoords(0,0,600,600)
    
    # create 4 bugs at various places in the window (using an appropriate
    # function)
    # put the bugs in a list

    bug1 = buildBug(Point(50,60),30)
    bug2 = buildBug(Point(200,350), 50)
    bug3 = buildBug(Point(350,50), 20)
    bug4 = buildBug(Point(350,300), 60)

    bugs = [bug1, bug2, bug3, bug4] # bug is defined as a list of the 4 bugs
    
    # draw the bugs (using an appropriate function)
    
    for n in range(4):
        drawBug(bugs[n], window)

    window.getMouse()

    # rotate one bug, pause

    for i in range(5):
        rotateBug(bug2,30)
        sleep(0.05)
        
        
    # change the color of one bug    
    for i in range(4):
        setBugColor(bugs[i], color_rgb(i*255/4, 255, 255))
        
    # wait for the user to click, then close the window
    window.getMouse()
    window.close()
   
main()


